# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Dataset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="Dataset.proto",
    package="",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b"\n\rDataset.proto\"\xdc\x05\n\x12\x44\x61tasetApiProtobuf\x12\x30\n\ngeometries\x18\x01 \x03(\x0b\x32\x1c.DatasetApiProtobuf.Geometry\x1a\xed\x01\n\x08Geometry\x12\x0e\n\x06\x64omain\x18\x01 \x01(\t\x12\x0c\n\x04lats\x18\x02 \x03(\x02\x12\x0c\n\x04lons\x18\x03 \x03(\x02\x12\x0c\n\x04\x61sls\x18\x04 \x03(\x02\x12\x15\n\rlocationNames\x18\x05 \x03(\t\x12\n\n\x02nx\x18\x06 \x01(\x12\x12\n\n\x02ny\x18\x07 \x01(\x12\x12\x16\n\x0etimeResolution\x18\x08 \x01(\t\x12\x37\n\rtimeIntervals\x18\t \x03(\x0b\x32 .DatasetApiProtobuf.TimeInterval\x12'\n\x05\x63odes\x18\n \x03(\x0b\x32\x18.DatasetApiProtobuf.Code\x1aO\n\x0cTimeInterval\x12\x13\n\x0btimestrings\x18\x01 \x03(\t\x12\r\n\x05start\x18\x02 \x01(\x12\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x12\x12\x0e\n\x06stride\x18\x04 \x01(\x03\x1a\xa6\x02\n\x04\x43ode\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x03\x12\r\n\x05level\x18\x02 \x01(\t\x12\x10\n\x08variable\x18\x03 \x01(\t\x12\x0c\n\x04unit\x18\x04 \x01(\t\x12\x13\n\x0b\x61ggregation\x18\x05 \x01(\t\x12/\n\rtimeIntervals\x18\x06 \x03(\x0b\x32\x18.DatasetApiProtobuf.Time\x12\x0f\n\x07gddBase\x18\x07 \x01(\x02\x12\x10\n\x08gddLimit\x18\x08 \x01(\x02\x12\r\n\x05slope\x18\t \x01(\x02\x12\x0e\n\x06\x66\x61\x63ing\x18\n \x01(\x02\x12\x0e\n\x06kwPeak\x18\x0b \x01(\x02\x12\x12\n\nstartDepth\x18\x0c \x01(\x03\x12\x10\n\x08\x65ndDepth\x18\r \x01(\x03\x12\x12\n\nefficiency\x18\x0e \x01(\x02\x12\x0f\n\x07tracker\x18\x0f \x01(\x03\x1a*\n\x04Time\x12\x14\n\x0cgapFillRatio\x18\x01 \x01(\x02\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x02\x62\x06proto3",
)


_DATASETAPIPROTOBUF_GEOMETRY = _descriptor.Descriptor(
    name="Geometry",
    full_name="DatasetApiProtobuf.Geometry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="domain",
            full_name="DatasetApiProtobuf.Geometry.domain",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="lats",
            full_name="DatasetApiProtobuf.Geometry.lats",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="lons",
            full_name="DatasetApiProtobuf.Geometry.lons",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="asls",
            full_name="DatasetApiProtobuf.Geometry.asls",
            index=3,
            number=4,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="locationNames",
            full_name="DatasetApiProtobuf.Geometry.locationNames",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="nx",
            full_name="DatasetApiProtobuf.Geometry.nx",
            index=5,
            number=6,
            type=18,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ny",
            full_name="DatasetApiProtobuf.Geometry.ny",
            index=6,
            number=7,
            type=18,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="timeResolution",
            full_name="DatasetApiProtobuf.Geometry.timeResolution",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="timeIntervals",
            full_name="DatasetApiProtobuf.Geometry.timeIntervals",
            index=8,
            number=9,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="codes",
            full_name="DatasetApiProtobuf.Geometry.codes",
            index=9,
            number=10,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=91,
    serialized_end=328,
)

_DATASETAPIPROTOBUF_TIMEINTERVAL = _descriptor.Descriptor(
    name="TimeInterval",
    full_name="DatasetApiProtobuf.TimeInterval",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="timestrings",
            full_name="DatasetApiProtobuf.TimeInterval.timestrings",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="start",
            full_name="DatasetApiProtobuf.TimeInterval.start",
            index=1,
            number=2,
            type=18,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="end",
            full_name="DatasetApiProtobuf.TimeInterval.end",
            index=2,
            number=3,
            type=18,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="stride",
            full_name="DatasetApiProtobuf.TimeInterval.stride",
            index=3,
            number=4,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=330,
    serialized_end=409,
)

_DATASETAPIPROTOBUF_CODE = _descriptor.Descriptor(
    name="Code",
    full_name="DatasetApiProtobuf.Code",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="code",
            full_name="DatasetApiProtobuf.Code.code",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="level",
            full_name="DatasetApiProtobuf.Code.level",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="variable",
            full_name="DatasetApiProtobuf.Code.variable",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="unit",
            full_name="DatasetApiProtobuf.Code.unit",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="aggregation",
            full_name="DatasetApiProtobuf.Code.aggregation",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="timeIntervals",
            full_name="DatasetApiProtobuf.Code.timeIntervals",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="gddBase",
            full_name="DatasetApiProtobuf.Code.gddBase",
            index=6,
            number=7,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="gddLimit",
            full_name="DatasetApiProtobuf.Code.gddLimit",
            index=7,
            number=8,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="slope",
            full_name="DatasetApiProtobuf.Code.slope",
            index=8,
            number=9,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="facing",
            full_name="DatasetApiProtobuf.Code.facing",
            index=9,
            number=10,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="kwPeak",
            full_name="DatasetApiProtobuf.Code.kwPeak",
            index=10,
            number=11,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="startDepth",
            full_name="DatasetApiProtobuf.Code.startDepth",
            index=11,
            number=12,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="endDepth",
            full_name="DatasetApiProtobuf.Code.endDepth",
            index=12,
            number=13,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="efficiency",
            full_name="DatasetApiProtobuf.Code.efficiency",
            index=13,
            number=14,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="tracker",
            full_name="DatasetApiProtobuf.Code.tracker",
            index=14,
            number=15,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=412,
    serialized_end=706,
)

_DATASETAPIPROTOBUF_TIME = _descriptor.Descriptor(
    name="Time",
    full_name="DatasetApiProtobuf.Time",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="gapFillRatio",
            full_name="DatasetApiProtobuf.Time.gapFillRatio",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="DatasetApiProtobuf.Time.data",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=708,
    serialized_end=750,
)

_DATASETAPIPROTOBUF = _descriptor.Descriptor(
    name="DatasetApiProtobuf",
    full_name="DatasetApiProtobuf",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="geometries",
            full_name="DatasetApiProtobuf.geometries",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _DATASETAPIPROTOBUF_GEOMETRY,
        _DATASETAPIPROTOBUF_TIMEINTERVAL,
        _DATASETAPIPROTOBUF_CODE,
        _DATASETAPIPROTOBUF_TIME,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=18,
    serialized_end=750,
)

_DATASETAPIPROTOBUF_GEOMETRY.fields_by_name[
    "timeIntervals"
].message_type = _DATASETAPIPROTOBUF_TIMEINTERVAL
_DATASETAPIPROTOBUF_GEOMETRY.fields_by_name[
    "codes"
].message_type = _DATASETAPIPROTOBUF_CODE
_DATASETAPIPROTOBUF_GEOMETRY.containing_type = _DATASETAPIPROTOBUF
_DATASETAPIPROTOBUF_TIMEINTERVAL.containing_type = _DATASETAPIPROTOBUF
_DATASETAPIPROTOBUF_CODE.fields_by_name[
    "timeIntervals"
].message_type = _DATASETAPIPROTOBUF_TIME
_DATASETAPIPROTOBUF_CODE.containing_type = _DATASETAPIPROTOBUF
_DATASETAPIPROTOBUF_TIME.containing_type = _DATASETAPIPROTOBUF
_DATASETAPIPROTOBUF.fields_by_name[
    "geometries"
].message_type = _DATASETAPIPROTOBUF_GEOMETRY
DESCRIPTOR.message_types_by_name["DatasetApiProtobuf"] = _DATASETAPIPROTOBUF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatasetApiProtobuf = _reflection.GeneratedProtocolMessageType(
    "DatasetApiProtobuf",
    (_message.Message,),
    {
        "Geometry": _reflection.GeneratedProtocolMessageType(
            "Geometry",
            (_message.Message,),
            {
                "DESCRIPTOR": _DATASETAPIPROTOBUF_GEOMETRY,
                "__module__": "Dataset_pb2"
                # @@protoc_insertion_point(class_scope:DatasetApiProtobuf.Geometry)
            },
        ),
        "TimeInterval": _reflection.GeneratedProtocolMessageType(
            "TimeInterval",
            (_message.Message,),
            {
                "DESCRIPTOR": _DATASETAPIPROTOBUF_TIMEINTERVAL,
                "__module__": "Dataset_pb2"
                # @@protoc_insertion_point(class_scope:DatasetApiProtobuf.TimeInterval)
            },
        ),
        "Code": _reflection.GeneratedProtocolMessageType(
            "Code",
            (_message.Message,),
            {
                "DESCRIPTOR": _DATASETAPIPROTOBUF_CODE,
                "__module__": "Dataset_pb2"
                # @@protoc_insertion_point(class_scope:DatasetApiProtobuf.Code)
            },
        ),
        "Time": _reflection.GeneratedProtocolMessageType(
            "Time",
            (_message.Message,),
            {
                "DESCRIPTOR": _DATASETAPIPROTOBUF_TIME,
                "__module__": "Dataset_pb2"
                # @@protoc_insertion_point(class_scope:DatasetApiProtobuf.Time)
            },
        ),
        "DESCRIPTOR": _DATASETAPIPROTOBUF,
        "__module__": "Dataset_pb2"
        # @@protoc_insertion_point(class_scope:DatasetApiProtobuf)
    },
)
_sym_db.RegisterMessage(DatasetApiProtobuf)
_sym_db.RegisterMessage(DatasetApiProtobuf.Geometry)
_sym_db.RegisterMessage(DatasetApiProtobuf.TimeInterval)
_sym_db.RegisterMessage(DatasetApiProtobuf.Code)
_sym_db.RegisterMessage(DatasetApiProtobuf.Time)


# @@protoc_insertion_point(module_scope)



""""
meteoblue dataset client
"""

import aiohttp
import asyncio
import logging
from contextlib import asynccontextmanager
from .Dataset_pb2 import DatasetApiProtobuf
from .utils import run_async


class ClientConfig(object):
    def __init__(self, apikey: str):
        # urls
        self.statusUrl = "http://my.meteoblue.com/queue/status/%s"  # following job id
        # following api key
        self.queryUrl = "http://my.meteoblue.com/dataset/query?apikey=%s"
        self.resultUrl = "http://queueresults.meteoblue.com/%s"  # following job id

        # http
        self.httpMaxRetryCount = 5
        self.httpRetrySleepDuration = 1

        # other config
        self.apikey = apikey

        self.queueRetrySleepDuration = 5


class Error(Exception):
    """Base class for exceptions in this module."""

    pass


class ApiError(Error):
    """Exception raised for errors by the dataset API

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message


class Client(object):
    def __init__(self, apikey: str):
        self._config = ClientConfig(apikey)

    @asynccontextmanager
    async def _fetch(
        self,
        session: aiohttp.ClientSession,
        method: str,
        url: str,
        retryCount=0,
        json: any = None,
    ):
        """
        Fetch data from an URL and try for error 5xx or timeouts.
        Codes other than 2xx will throw an exception.

        :param url: url to make http get on
        :param retryCount: number of retries to attempt
        :return:
        """
        logging.debug("Getting url %s %s" % (method, url))

        for retry in range(self._config.httpMaxRetryCount):
            async with session.request(method, url, json=json) as response:
                # return if successful
                if 200 <= response.status <= 299:
                    yield response
                    return

                # meteoblue APIs return a JSON encoded error message
                if response.status == 400 or response.status == 500:
                    json = await response.json()
                    logging.debug(
                        "API returned error message: %s" % json["error_message"]
                    )
                    raise ApiError(json["error_message"])

                if retry == self._config.httpMaxRetryCount - 1:
                    logging.error(
                        "API returned unexpected error: %s" % response.content
                    )
                    raise Exception("API returned unexpected error", response.content)

                # retry mechanism
                # check if request timed out or backend threw an error
                # if response.status == 408 or 500 <= response.status <= 599:
                logging.info("Request failed or timed out. Try %s" % retry)
                logging.debug(response)

    @asynccontextmanager
    async def _runOnJobQueue(self, session: aiohttp.ClientSession, params: dict):
        """
        Run the query on a job queue
        :param session: ClientSession
        :param params: query parameters for meteoblue dataset api
        :return: ClientResponse object from aiohttp lib
        """

        # Start job on a job queue
        logging.info("Starting job on queue")
        params["runOnJobQueue"] = True
        url = self._config.queryUrl % self._config.apikey
        async with self._fetch(session, "POST", url, json=params) as response:
            responseJson = await response.json()

        # Wait until the job is finished
        jobId = responseJson["id"]
        logging.info("Waiting until job has finished (job id %s)" % jobId)
        statusUrl = self._config.statusUrl % str(jobId)
        while True:
            async with self._fetch(session, "GET", statusUrl) as response:
                json = await response.json()
            status = json["status"]
            logging.debug("Job status is %s" % status)
            if status == "finished":
                break
            if status == "deleted":
                raise ApiError("Job was canceled")
            if status == "error":
                raise ApiError(json["error_message"])
            logging.info(
                "Waiting 5 seconds for job to complete. Status: %s, job id %s"
                % (status, jobId)
            )
            await asyncio.sleep(self._config.queueRetrySleepDuration)

        # Fetch the job queue result
        resultUrl = self._config.resultUrl % jobId
        logging.debug("Fetching result for job id %s" % jobId)
        async with self._fetch(session, "GET", resultUrl) as response:
            yield response

    @asynccontextmanager
    async def queryRaw(self, params: dict):
        """
        Query meteoblue dataset api asynchronously and return a ClientResponse object using context manager
        :param params: query parameters, see https://docs.meteoblue.com/en/apis/environmental-data/dataset-api
        :return: ClientResponse object from aiohttp lib
        """

        async with aiohttp.ClientSession() as session:
            # Try to run the job directly
            # In case the API throws an error, try to run it on a job queue
            try:
                url = self._config.queryUrl % self._config.apikey
                async with self._fetch(session, "POST", url, json=params) as response:
                    yield response
            except ApiError as error:
                # Run on a job queue in case the api throws the error
                if error.message != "This job must be executed on a job-queue":
                    raise
                async with self._runOnJobQueue(session, params) as response:
                    yield response

    async def query(self, params: dict):
        """
        Query meteoblue dataset api asynchronously, transfer data using protobuf and return a structured object
        :param params: query parameters, see https://docs.meteoblue.com/en/apis/environmental-data/dataset-api
        :return: DatasetApiProtobuf object
        """

        params["format"] = "protobuf"

        async with self.queryRaw(params) as response:
            data = await response.read()
            msg = DatasetApiProtobuf()
            msg.ParseFromString(data)
            return msg

    def querySync(self, params: dict):
        """
        Query meteoblue dataset api synchronously for sequential usage
        :param params: query parameters, see https://docs.meteoblue.com/en/apis/environmental-data/dataset-api
        :return: DatasetApiProtobuf object
        """
        return run_async(self.query, params)



import nox


@nox.session
def lint(session):
    lint_tools = ["flake8"]  # "black", "isort",
    targets = ["meteoblue_dataset_sdk", "tests", "noxfile.py"]
    session.install(*lint_tools)
    session.run("flake8", *targets)
    # session.run("black", "--diff", "--check", *targets)
    # session.run("isort", "--check-only", *targets)


@nox.session
def tests(session):
    session.install(".[test]")
    session.run("pytest", "-v")
